# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.pydantic_utilities import pydantic_v1
from ...entity_types.types.notification_type import NotificationType


class NotificationConfigurationResponse_Invoice(pydantic_v1.BaseModel):
    notification_type: typing.Literal["invoice"] = pydantic_v1.Field(alias="notificationType", default="invoice")
    type: NotificationType
    url: str

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


NotificationConfigurationResponse = typing.Union[NotificationConfigurationResponse_Invoice]
