# This file was auto-generated by Fern from our API Definition.

import typing

EntityStatus = typing.Union[
    typing.Literal["unverified", "pending", "resubmit", "review", "verified", "failed"], typing.Any
]
