# This file was auto-generated by Fern from our API Definition.

import typing

BusinessType = typing.Union[
    typing.Literal[
        "soleProprietorship",
        "unincorporatedAssociation",
        "trust",
        "publicCorporation",
        "privateCorporation",
        "llc",
        "partnership",
        "unincorporatedNonProfit",
        "incorporatedNonProfit",
    ],
    typing.Any,
]
