# This file was auto-generated by Fern from our API Definition.

import typing

PaymentMethodType = typing.Union[
    typing.Literal["custom", "check", "bankAccount", "na", "card", "bnpl", "virtualCard", "offPlatform"], typing.Any
]
