# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.pydantic_utilities import pydantic_v1
from .entity_user_id import EntityUserId


class IdentifierList_RolesList(pydantic_v1.BaseModel):
    type: typing.Literal["rolesList"] = "rolesList"
    value: typing.List[str]

    class Config:
        frozen = True
        smart_union = True


class IdentifierList_UserList(pydantic_v1.BaseModel):
    type: typing.Literal["userList"] = "userList"
    value: typing.List[EntityUserId]

    class Config:
        frozen = True
        smart_union = True


IdentifierList = typing.Union[IdentifierList_RolesList, IdentifierList_UserList]
