# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.pydantic_utilities import pydantic_v1
from .bank_delivery_method import BankDeliveryMethod
from .check_delivery_method import CheckDeliveryMethod


class PaymentDestinationOptions_Check(pydantic_v1.BaseModel):
    type: typing.Literal["check"] = "check"
    delivery: typing.Optional[CheckDeliveryMethod]

    class Config:
        frozen = True
        smart_union = True


class PaymentDestinationOptions_BankAccount(pydantic_v1.BaseModel):
    type: typing.Literal["bankAccount"] = "bankAccount"
    delivery: typing.Optional[BankDeliveryMethod]

    class Config:
        frozen = True
        smart_union = True


"""
from mercoa import PaymentDestinationOptions_Check

PaymentDestinationOptions_Check(
    delivery="MAIL",
)
"""
PaymentDestinationOptions = typing.Union[PaymentDestinationOptions_Check, PaymentDestinationOptions_BankAccount]
