# This file was auto-generated by Fern from our API Definition.

import typing

InvoiceFailureType = typing.Union[
    typing.Literal[
        "SOURCE_PAYMENT_ERROR",
        "DESTINATION_PAYMENT_ERROR",
        "REJECTED_HIGH_RISK",
        "INSUFFICIENT_FUNDS",
        "PROCESSING_ERROR",
    ],
    typing.Any,
]
