# This file was auto-generated by Fern from our API Definition.

import typing

InvoiceStatus = typing.Union[
    typing.Literal[
        "DRAFT", "NEW", "APPROVED", "SCHEDULED", "PENDING", "PAID", "ARCHIVED", "REFUSED", "CANCELED", "FAILED"
    ],
    typing.Any,
]
