# This file was auto-generated by Fern from our API Definition.

import typing

InvoiceMetricsPerDateGroupBy = typing.Union[
    typing.Literal["CREATION_DATE", "DUE_DATE", "INVOICE_DATE", "DEDUCTION_DATE"], typing.Any
]
