# This file was auto-generated by Fern from our API Definition.

import typing

CustomPaymentMethodSchemaFieldType = typing.Union[
    typing.Literal["text", "number", "select", "date", "phone", "email", "url", "address"], typing.Any
]
