# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import pydantic_v1
from .notification_type import NotificationType


class UserNotificationPolicyResponse(pydantic_v1.BaseModel):
    """
    Examples
    --------
    from mercoa import UserNotificationPolicyResponse

    UserNotificationPolicyResponse(
        disabled=False,
        digest=True,
        immediate=False,
        type="INVOICE_APPROVAL_NEEDED",
    )
    """

    disabled: bool = pydantic_v1.Field()
    """
    True if the selected notification type is disabled for this user
    """

    digest: bool = pydantic_v1.Field()
    """
    True if the selected notification type is sent as a digest.
    """

    immediate: bool = pydantic_v1.Field()
    """
    True if the selected notification type is sent immediately.
    """

    type: NotificationType

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}
